@@ -10,11 +10,11 @@ Interchange6::Schema - Database Schema for Interchange 6
 
 =head1 VERSION
 
-0.101
+0.102
 
 =cut
 
-our $VERSION = '0.101';
+our $VERSION = '0.102';
 
 =head1 MANUAL
 
@@ -27,6 +27,16 @@ Database schema classes for Interchange6 Open Source eCommerce software.
 
 The minimum Perl version for Interchange6::Schema is 5.8.
 
+=head1 BASE CLASS
+
+L<DBIx::Class::Schema::Config>
+
+If you inherit from L<Interchange6::Schema> then in order to make use of
+L<DBIx::Class::Schema::Config> in your own schema you must load it as a
+component.
+
+=head1 COMPONENTS
+
 Components used:
 
 =over
@@ -81,14 +91,14 @@ This attribute can be used to store the locale of the current user.
 =cut
 
 __PACKAGE__->mk_group_ro_accessors(
-    simple => (
+    inherited => (
         [ 'current_user' => '_ic6_current_user' ],
         [ 'user_locale'  => '_ic6_user_locale' ],
     )
 );
 
 __PACKAGE__->mk_group_wo_accessors(
-    simple => (
+    inherited => (
         [ 'set_current_user' => '_ic6_current_user' ],
         [ 'set_user_locale'  => '_ic6_user_locale' ],
     )
