@@ -28,7 +28,7 @@ FK on L<Interchange6::Schema::Result::Zone/zones_id>.
 =cut
 
 column zones_id =>
-  { data_type => "integer", is_foreign_key => 1 };
+  { data_type => "integer" };
 
 =head2 shipment_methods_id
 
@@ -37,7 +37,7 @@ FK on L<Interchange6::Schema::Result::ShipmentMethod/shipment_methods_id>.
 =cut
 
 column shipment_methods_id =>
-  { data_type => "integer", is_foreign_key => 1 };
+  { data_type => "integer" };
 
 =head2 active
 
