@@ -31,7 +31,7 @@ Returns all rows where L<Interchange6::Schema::Result::Product/active> is true.
 =cut
 
 sub active {
-    return $_[0]->search({ $_[0]->me.'active' => 1 });
+    return $_[0]->search({ $_[0]->me('active') => 1 });
 }
 
 =head2 canonical_only
@@ -42,7 +42,7 @@ is null, i.e. only canonical products.
 =cut
 
 sub canonical_only {
-    return $_[0]->search({ $_[0]->me.'canonical_sku' => undef });
+    return $_[0]->search({ $_[0]->me('canonical_sku') => undef });
 }
 
 =head2 listing
@@ -59,13 +59,13 @@ This is just a shortcut for:
 Though in addition if you pass in arguments these are passed through to the
 appropriate with_* method so you can do:
 
-  $self->listing({ quantity => 10, users_id => 43 })
+  $self->listing({ quantity => 10 })
 
 And the result will be:
 
   $self->columns( [ 'sku', 'name', 'uri', 'price', 'short_description' ] )
       ->with_average_rating
-      ->with_lowest_selling_price({ quantity => 10, users_id => 43 })
+      ->with_lowest_selling_price({ quantity => 10 })
       ->with_highest_price
       ->with_quantity_in_stock
       ->with_variant_count
@@ -79,8 +79,6 @@ sub listing {
       ->with_average_rating->with_lowest_selling_price(
         {
             quantity => $args->{quantity},
-            users_id => $args->{users_id},
-            roles    => $args->{roles}
         }
       )->with_highest_price->with_quantity_in_stock->with_variant_count;
 }
@@ -100,8 +98,6 @@ products so for variants the value returned is that of the canonical product.
 sub with_average_rating {
     my $self = shift;
 
-    my $me = $self->me;
-
     return $self->search(
         undef,
         {
@@ -110,16 +106,20 @@ sub with_average_rating {
                     coalesce => [
 
                         $self->correlate('canonical')
-                          ->related_resultset('_product_reviews')
+                          ->related_resultset('product_messages')
                           ->search_related(
                             'message',
-                            { 'message.approved' => 1, 'message.public' => 1 }
+                            { 'message.approved' => 1, 'message.public' => 1,
+                           'message_type.name' => 'product_review' },
+                            { join => 'message_type' },
                           )->get_column('rating')->func_rs('avg')->as_query,
 
-                        $self->correlate('_product_reviews')
+                        $self->correlate('product_messages')
                           ->search_related(
                             'message',
-                            { 'message.approved' => 1, 'message.public' => 1 }
+                            { 'message.approved' => 1, 'message.public' => 1,
+                           'message_type.name' => 'product_review' },
+                            { join => 'message_type' },
                           )->get_column('rating')->func_rs('avg')->as_query,
 
                       ],
@@ -131,6 +131,31 @@ sub with_average_rating {
     );
 }
 
+=head2 with_media $type?
+
+Prefetch related active L<Interchange6::Schema::Result::Media> where
+L<Interchange6::Schema::Result::MediaType/type> is C<$type>.
+
+C<$type> defaults to C<image> if not provided.
+
+=cut
+
+sub with_media {
+    my $self = shift;
+    my $type = defined $_[0] ? $_[0] : 'image';
+
+    return $self->search(
+        {
+            'media.active'    => 1,
+            'media_type.type' => $type,
+        },
+        {
+            prefetch => { media_products => 'media' },
+            join     => { media_products => { media => 'media_type' } },
+        }
+    );
+}
+
 =head2 with_quantity_in_stock
 
 Adds C<quantity_in_stock> column which is available to order_by clauses and
@@ -175,6 +200,16 @@ sub with_quantity_in_stock {
 
 =head2 with_lowest_selling_price
 
+Arguments should be given as a hash reference with the following keys/values:
+
+=over 4
+
+=item * quantity => $quantity
+
+C<quantity> defaults to 1 if not supplied.
+
+=back
+
 The lowest of L<Interchange6::Schema::Result::PriceModifier/price> and
 L<Interchange6::Schema::Result::Product/price>.
 
@@ -182,6 +217,10 @@ For products with variants this is the lowest variant selling_price.
 
 Value is placed in the column C<selling_price>.
 
+If L<Schema/current_user> is defined then any roles assigned to that
+user will be included in the search of
+L<Interchange6::Schema::Result::PriceModifier>.
+
 =cut
 
 sub with_lowest_selling_price {
@@ -189,43 +228,41 @@ sub with_lowest_selling_price {
 
     if ( defined($args) ) {
         $self->throw_exception(
-            "argument to listing must be a hash reference")
+            "argument to with_lowest_selling_price must be a hash reference")
           unless ref($args) eq "HASH";
     }
 
-    my $schema = $self->result_source->schema;
-
     $args->{quantity} = 1 unless defined $args->{quantity};
 
-    my $roles_id = undef;
-    my @roles_cond = ( undef );
-
-    if ( $args->{users_id} ) {
-
-        my $subquery =
-          $schema->resultset('UserRole')
-          ->search( { "role.users_id" => $args->{users_id} },
-            { alias => 'role' } )->get_column('roles_id')->as_query;
-
-        push @roles_cond, { -in => $subquery };
-    }
-
-    if ( $args->{roles} ) {
-
-        $self->throw_exception(
-            "Argument roles to selling price must be an array reference")
-          unless ref( $args->{roles} ) eq 'ARRAY';
-
-        my $subquery =
-          $schema->resultset('Role')
-          ->search( { "role.name" => { -in => \@{$args->{roles}} } },
-            { alias => 'role' } )->get_column('roles_id')->as_query;
-
-        push @roles_cond, { -in => $subquery };
-    }
+    my $schema = $self->result_source->schema;
 
     my $today = $schema->format_datetime(DateTime->today);
 
+    # start building the search condition
+
+    my $search_cond = {
+        'start_date' => [ undef, { '<=', $today } ],
+        'end_date'   => [ undef, { '>=', $today } ],
+        'quantity'   => { '<=' => $args->{quantity} },
+        'roles_id'   => undef,
+    };
+
+    if ( my $user = $schema->current_user ) {
+
+        # add roles_id condition
+
+        $search_cond->{roles_id} = [
+            undef,
+            {
+                -in => $schema->resultset('UserRole')
+                  ->search( { users_id => $user->id } )
+                  ->get_column('roles_id')->as_query
+            }
+        ];
+    }
+
+    # most db engines have 'least' but SQLite has 'min'
+
     my $least = 'least';
     $least = 'min' if $schema->storage->sqlt_type eq 'SQLite';
 
@@ -244,13 +281,6 @@ sub with_lowest_selling_price {
     # NULL values in the list are ignored. The result will be NULL only if all
     # the expressions evaluate to NULL.
 
-    my $search_cond = {
-        'start_date' => [ undef, { '<=', $today } ],
-        'end_date'   => [ undef, { '>=', $today } ],
-        'quantity'   => { '<=' => $args->{quantity} },
-        'roles_id'   => \@roles_cond,
-    };
-
     my $variant_price_modifiers =
       $self->correlate('variants')
       ->search_related( 'price_modifiers', $search_cond )->get_column('price')
@@ -263,7 +293,7 @@ sub with_lowest_selling_price {
       $self->correlate('price_modifiers')->search( $search_cond )
       ->get_column('price')->min_rs->as_query;
 
-    return $self->search(
+    my $return = $self->search(
         undef,
         {
             '+select' => [
@@ -292,6 +322,7 @@ sub with_lowest_selling_price {
             '+as' => ['selling_price'],
         }
     );
+    return $return;
 }
 
 =head2 with_highest_price
@@ -308,9 +339,6 @@ sub with_highest_price {
 
     my $schema = $self->result_source->schema;
 
-    my $greatest = 'greatest';
-    $greatest = 'max' if $schema->storage->sqlt_type eq 'SQLite';
-
     return $self->search(
         undef,
         {
