@@ -38,9 +38,14 @@ test 'product tests' => sub {
 
         lives_ok(
             sub {
-                $product =
-                  $products->create(
-                    { name => $key, sku => ++$sku, description => '' } );
+                $product = $products->create(
+                    {
+                        name        => $key,
+                        sku         => ++$sku,
+                        description => '',
+                        price       => 1
+                    }
+                );
             },
             "create product for name: " . Encode::encode_utf8($key)
         );
@@ -73,7 +78,8 @@ test 'product tests' => sub {
                 {
                     name        => 'one banana and a carrot',
                     sku         => '1001',
-                    description => ''
+                    description => '',
+                    price       => 1,
                 }
             );
         },
@@ -109,7 +115,8 @@ test 'product tests' => sub {
                 {
                     name        => 'One BANANA and a carrot',
                     sku         => '1002',
-                    description => ''
+                    description => '',
+                    price       => 1,
                 }
             );
         },
@@ -145,7 +152,8 @@ test 'product tests' => sub {
                 {
                     name        => 'One BANANA and a carrot',
                     sku         => '1003',
-                    description => ''
+                    description => '',
+                    price       => 1,
                 }
             );
         },
@@ -211,35 +219,125 @@ test 'product tests' => sub {
         cmp_ok( $product->average_rating, '==', 4.3, "average_rating is 4.3" );
         cmp_ok( $product->average_rating(2),
             '==', 4.27, "average_rating to 2 DP is 4.27" );
+        cmp_ok( $product->average_rating('q'),
+            '==', 4.3,
+            "average_rating('q') ignores pricision 'q' so gives 4.3 to 1 DP" );
 
         $i = 1;
     }
 
     # quantity_in_stock
 
+    # we need inventory & be paranoid about it
+    $self->clear_inventory;
+    $self->inventory;
+
     lives_ok( sub { $products = $self->products->with_quantity_in_stock },
         "get products with_quantity_in_stock" );
 
     cmp_ok( $products->count, '==', $num_products, "$num_products products" );
 
+    $i = 0;
+    while ( my $product = $products->next ) {
+        my $qis = $product->quantity_in_stock;
+        my $inventory = $product->inventory;
+        if ( defined $inventory ) {
+            $i++;
+            cmp_ok $qis, '==', $inventory->quantity,
+              "quantity_in_stock == inventory->quantity for: "
+              . $product->sku;
+        }
+    }
+    cmp_ok $i, '==', 59, "found $i/59 products with quantity_in_stock";
+
+    lives_ok {
+        $product = $self->products->create(
+            {
+                name        => "parent",
+                sku         => "parent",
+                description => "parent",
+                price       => 1,
+                variants    => [
+                    {
+                        name             => "variant",
+                        sku              => "variant",
+                        description      => "variant",
+                        inventory_exempt => 1,
+                        price            => 1,
+                        inventory        => { quantity => 1 },
+                    },
+                ],
+            }
+        );
+    }
+    "Create product with inventory_exempt variant";
+
+    ok !defined $product->quantity_in_stock, "quantity_in_stock is undef";
+
+    lives_ok { $product->variants->delete } "delete the variant";
+    lives_ok { $product->delete } "delete the product";
+
     # lowest_selling_price
 
+    throws_ok(
+        sub { $products = $self->products->with_lowest_selling_price('foo') },
+        qr/argument to with_lowest_selling_price must be a hash reference/,
+        "with_lowest_selling_price with bad arg fails"
+    );
+
+    lives_ok(
+        sub {
+            $products =
+              $self->products->with_lowest_selling_price(
+                { roles => ['user'] } );
+        },
+        "get products with_lowest_selling_price({roles => ['user']})"
+    );
+
     lives_ok( sub { $products = $self->products->with_lowest_selling_price },
         "get products with_lowest_selling_price" );
 
     cmp_ok( $products->count, '==', $num_products, "$num_products products" );
 
+    throws_ok { $self->products->with_lowest_selling_price( "badarg" ) }
+    qr/argument to with_lowest_selling_price must be a hash reference/,
+    "call with_lowest_selling_price on products with bad arg";
+
+    lives_ok {
+        $products =
+          $self->products->with_lowest_selling_price( { roles => ['user'] } )
+    }
+    "with_lowest_selling_price OK with single role";
+
+    lives_ok {
+        $products =
+          $self->products->with_lowest_selling_price(
+            { roles => [ 'user', 'admin' ] } )
+    }
+    "with_lowest_selling_price OK with two roles";
+
     lives_ok( sub { $product = $products->find('os28006') },
         "get product os28006" );
 
     cmp_deeply( $product->price, num(29.99, 0.01), "price is 29.99" );
 
+    throws_ok { $product->selling_price(12) }
+    qr/Argument to selling_price must be a hash reference/,
+      "selling_price with bad arg throws exception";
+
+    throws_ok { $product->selling_price( { quantity => 'q' } ) }
+    qr/Bad quantity/,
+      "selling_price with bad quantity throws exception";
+
     cmp_deeply(
         $product->selling_price,
         num( 24.99, 0.01 ),
         "selling_price is 24.99"
     );
 
+    lives_ok { $product->selling_price( { roles => ['user'] } ) }
+      "selling_price with good roles";
+
     cmp_ok( $product->discount_percent, '==', 17, "discount_percent is 17" );
 
     lives_ok( sub { $product = $products->find('os28085') },
@@ -300,6 +398,8 @@ test 'product tests' => sub {
         cmp_ok( $product->variant_count,
             '==', 2, "product variant_count is 2" );
 
+        ok $product->has_variants, "product has_variants is true";
+
         lives_ok( sub { $product = $rset->find('os28085-12') },
             "get product os28085-12" );
 
@@ -365,7 +465,10 @@ test 'product tests' => sub {
                     "get variants with_lowest_selling_price"
                 );
                 while ( my $variant = $variants_min_rset->next ) {
-                    my $min = $variant->selling_price // $variant->price;
+                    my $min =
+                      defined $variant->selling_price
+                      ? $variant->selling_price
+                      : $variant->price;
                     unless ( $variants_min ) {
                         $variants_min = $min;
                         next;
